var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { AbstractSDKRequest, StorageType, } from '@cloudbase/adapter-interface';
import * as fs from 'fs';
import * as path from 'path';
import { remote } from 'electron';
var PROTOCOL = typeof location !== 'undefined' && location.protocol === 'http:'
    ? 'http:'
    : 'https:';
function formatUrl(PROTOCOL, url, query) {
    if (query === void 0) { query = {}; }
    var urlHasQuery = /\?/.test(url);
    var queryString = '';
    for (var key in query) {
        if (queryString === '') {
            !urlHasQuery && (url += '?');
        }
        else {
            queryString += '&';
        }
        queryString += key + "=" + encodeURIComponent(query[key]);
    }
    url += queryString;
    if (/^http(s)?\:\/\//.test(url)) {
        return url;
    }
    return "" + PROTOCOL + url;
}
function isFormData(val) {
    return Object.prototype.toString.call(val) === '[object FormData]';
}
function toQueryString(query) {
    if (query === void 0) { query = {}; }
    var queryString = [];
    for (var key in query) {
        queryString.push(key + "=" + encodeURIComponent(query[key]));
    }
    return queryString.join('&');
}
function isMatch() {
    if (!window || typeof window !== 'object') {
        return false;
    }
    if (!window.fetch || typeof window.fetch !== 'function') {
        return false;
    }
    if (!WebSocket || typeof WebSocket !== 'function') {
        return false;
    }
    if (!localStorage || typeof localStorage !== 'object') {
        return false;
    }
    if (!sessionStorage || typeof sessionStorage !== 'object') {
        return false;
    }
    if (!fs || !path || !remote) {
        return false;
    }
    return true;
}
var SDKRequestImpl = (function (_super) {
    __extends(SDKRequestImpl, _super);
    function SDKRequestImpl(config) {
        if (config === void 0) { config = {}; }
        var _this = _super.call(this) || this;
        var timeout = config.timeout, timeoutMsg = config.timeoutMsg, restrictedMethods = config.restrictedMethods;
        _this._timeout = timeout || 1000 * 20;
        _this._timeoutMsg = timeoutMsg || '请求超时';
        _this._restrictedMethods = restrictedMethods || ['get', 'post', 'upload', 'download'];
        return _this;
    }
    SDKRequestImpl.prototype._request_by_fetch = function (options) {
        var _a;
        var method = ((_a = options.method) === null || _a === void 0 ? void 0 : _a.toUpperCase()) || 'GET';
        return new Promise(function (resolve, reject) {
            var url = options.url, data = options.data, body = options.body, _a = options.headers, headers = _a === void 0 ? {} : _a;
            var payload = '';
            var contentType = headers["content-type"] || headers["Content-Type"] || "";
            if (isFormData(data)) {
                payload = data;
            }
            else if (contentType === 'application/x-www-form-urlencoded') {
                payload = toQueryString(data);
            }
            else if (data) {
                payload = JSON.stringify(data);
            }
            else if (body) {
                payload = body;
            }
            var config = {
                method: method,
                headers: __assign({ mode: 'cors' }, headers),
                body: payload
            };
            if (['GET', 'HEAD'].includes(method)) {
                delete config.body;
            }
            fetch(url, config).then(function (response) {
                if (!response.ok)
                    reject(response);
                else {
                    try {
                        var conType = response.headers['content-type'] || response.headers['Content-Type'] || '';
                        if (conType === 'application/json') {
                            return response.json();
                        }
                        else if (conType.startsWith("image")) {
                            return response.blob();
                        }
                        else if (conType.startsWith("text") || conType === '') {
                            return response.text();
                        }
                        else {
                            return response.blob();
                        }
                    }
                    catch (err) {
                        console.error('response err', err);
                    }
                }
            })
                .then(function (data) {
                if (typeof data === 'string') {
                    try {
                        data = JSON.parse(data);
                    }
                    catch (err) { }
                }
                resolve({
                    status: 200,
                    statusCode: 200,
                    data: data,
                });
            });
        });
    };
    SDKRequestImpl.prototype._request_by_fetch_with_timeout = function (options) {
        var _this = this;
        return Promise.race([
            this._request_by_fetch(options),
            new Promise(function (resolve) {
                console.warn(_this._timeoutMsg);
                setTimeout(function () { return resolve({ msg: _this._timeoutMsg }); }, _this._timeout);
            })
        ]);
    };
    SDKRequestImpl.prototype._request_by_xhr = function (options, enableAbort) {
        var _this = this;
        if (enableAbort === void 0) { enableAbort = false; }
        var method = (String(options.method)).toLowerCase() || 'get';
        return new Promise(function (resolve) {
            var url = options.url, _a = options.headers, headers = _a === void 0 ? {} : _a, data = options.data, responseType = options.responseType, withCredentials = options.withCredentials, body = options.body, onUploadProgress = options.onUploadProgress;
            var realUrl = formatUrl(PROTOCOL, url, method === 'get' ? data : {});
            var ajax = new XMLHttpRequest();
            ajax.open(method, realUrl);
            responseType && (ajax.responseType = responseType);
            for (var key in headers) {
                ajax.setRequestHeader(key, headers[key]);
            }
            var timer;
            if (onUploadProgress) {
                ajax.addEventListener('progress', onUploadProgress);
            }
            ajax.onreadystatechange = function () {
                var result = {};
                if (ajax.readyState === 4) {
                    var headers_1 = ajax.getAllResponseHeaders();
                    var arr = headers_1.trim().split(/[\r\n]+/);
                    var headerMap_1 = {};
                    arr.forEach(function (line) {
                        var parts = line.split(': ');
                        var header = parts.shift().toLowerCase();
                        var value = parts.join(': ');
                        headerMap_1[header] = value;
                    });
                    result.header = headerMap_1;
                    result.statusCode = ajax.status;
                    try {
                        result.data = responseType === 'blob' ? ajax.response : JSON.parse(ajax.responseText);
                    }
                    catch (e) {
                        result.data = responseType === 'blob' ? ajax.response : ajax.responseText;
                    }
                    clearTimeout(timer);
                    resolve(result);
                }
            };
            if (enableAbort && _this._timeout) {
                timer = setTimeout(function () {
                    console.warn(_this._timeoutMsg);
                    ajax.abort();
                }, _this._timeout);
            }
            var payload;
            if (isFormData(data)) {
                payload = data;
            }
            else if (headers['content-type'] === 'application/x-www-form-urlencoded') {
                payload = toQueryString(data);
            }
            else if (body) {
                payload = body;
            }
            else {
                payload = data ? JSON.stringify(data) : undefined;
            }
            if (withCredentials) {
                ajax.withCredentials = true;
            }
            ajax.send(payload);
        });
    };
    SDKRequestImpl.prototype._request = function (options) {
        var _a;
        var method = ((_a = options.method) === null || _a === void 0 ? void 0 : _a.toLowerCase()) || 'get';
        var restricted = this._restrictedMethods.includes(method);
        return this._request_by_xhr(options, restricted);
    };
    SDKRequestImpl.prototype.post = function (options) {
        return this._request(__assign(__assign({}, options), { method: 'POST' }));
    };
    SDKRequestImpl.prototype.upload = function (options) {
        var data = options.data, file = options.file, name = options.name;
        var formData = new FormData();
        for (var key in data) {
            formData.append(key, data[key]);
        }
        formData.append('key', name);
        var fileContent = fs.readFileSync(file);
        formData.append('file', new File([fileContent], name));
        return this._request(__assign(__assign({}, options), { data: formData, method: 'POST' })).then(function (resp) {
            if (resp.statusCode === 200)
                resp.statusCode = 201;
            return resp;
        });
    };
    SDKRequestImpl.prototype.download = function (options) {
        return this._request(options).then(function (resp) {
            var _a;
            var userDataPath = remote.app.getPath('userData');
            var filePath = path.join(userDataPath, 'tmp');
            if (!fs.existsSync(filePath))
                fs.mkdirSync(filePath);
            var fileName = decodeURIComponent(((_a = options.url) === null || _a === void 0 ? void 0 : _a.split('/').pop()) || (new Date()).valueOf() + '');
            filePath = path.join(filePath, fileName);
            fs.writeFileSync(filePath, resp.data);
            return {
                code: 'SUCCESS',
                status: 200,
                statusCode: 200,
                tempFilePath: filePath
            };
        })
            .catch(function (err) {
            console.error(err);
            return err;
        });
    };
    return SDKRequestImpl;
}(AbstractSDKRequest));
function genAdapter() {
    var adapter = {
        root: window,
        localStorage: localStorage,
        sessionStorage: sessionStorage,
        reqClass: SDKRequestImpl,
        wsClass: WebSocket,
        primaryStorage: StorageType.local,
    };
    return adapter;
}
var adapter = {
    genAdapter: genAdapter,
    isMatch: isMatch,
    runtime: 'electron_renderer_process',
};
export { adapter };
export default adapter;
